/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.usercontrols;

import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.gui.quickview.QuickViewAction;

/**
 * An action that configures the Shrimp View to its default nested view.
 * The nested arc types (cprels) are determined by the dataBean, and all node and arc types are shown.
 * 
 * @author Rob Lintern, Chris Callendar
 */
public class DefaultViewAction extends QuickViewAction {
    
	public static final String ACTION_NAME = "Nested View (Default)";
	
	/**
	 * Don't use this constructor.  It is used to instantiate this class using reflection.
	 */
	public DefaultViewAction() {
		super();
	}	
	
	public DefaultViewAction(ShrimpProject project) {
	    this (ACTION_NAME, null, project);
	}

	public DefaultViewAction(ShrimpProject project, String iconFilename) {
	    this (ACTION_NAME, iconFilename, project);
	}
	
	public DefaultViewAction(String actionName, String iconFilename, ShrimpProject project) {
	    super(actionName, iconFilename, project);
	    
	    config.setNodeTypesOfInterest(getAllNodeTypes());
	    config.setArcTypesOfInterest(getAllArcTypes());
	    // Respect the default label mode set in application properties 
		String labelMode = 
			ApplicationAccessor.getApplication().getProperties().getProperty(
					ShrimpView.SV_LABEL_MODE_KEY);
	    config.setLabelMode(labelMode);
//	    config.setLabelMode(DisplayConstants.LABEL_MODE_FIXED);
	    config.setCprels(getDefaultCprels());
	    config.setInverted(getDefaultCprelsInverted());
	}
		
}
